@@ -4,8 +4,8 @@
 #' The function ei_gme defines the Shannon entropy function which takes a vector of probabilities as input and returns the negative
 #' sum of p times the natural logarithm of p.The function will set the optimization parameters and using the "nlminb" function an optimal
 #' solution is obtained.
-#' The function defines the independent variables in the two databases needed, which we call datahp with "n_hp" observations and datahs
-#' with "n_hs" observations; and the function of the binary variable of interest y. Then the weights of each observation for the two
+#' The function defines the independent variables in the two databases needed, which we call dataA with "n_A" observations and dataB
+#' with "n_B" observations; and the function of the binary variable of interest y. Then the weights of each observation for the two
 #' databases used are defined, if there are no weights available it will be 1.
 #' The errors are calculated pondering the support vector of dimension \code{var, 0, -var}. This support vector can be specified by the user.
 #' The default support vector is based on variance.We recommend a wider interval with v(1,0,-1) as the maximum.
@@ -21,17 +21,17 @@
 #'To solve the optimization upper and lower bounds for p and w are settled, specifically, p and w must be above 0 and lower than 1.
 #'In addition, the initial values of p are settled as a uniform distribution and the errors (w) as 1/L.
 #'
-#' @param fn is the formula that represents the dependent variable in the optimization.
+#' @param fn Is the formula that represents the dependent variable in the optimization.
 #' In the context of this function, 'fn' is used to define the dependent variable
 #' to be optimized by the entropy function.
 #' Note: If the dependent variable is categorical the sorting criterion for the columns, and therefore for J, is alphabetical order.
-#' @param datahp The data where the variable of interest y is available and also the independent variables.
-#'  Note: The variables and weights used as independent variables must have the same name in 'datahp' and in 'datahs'
+#' @param dataA The data where the variable of interest y is available and also the independent variables.
+#'  Note: The variables and weights used as independent variables must have the same name in 'dataA' and in 'dataB'
 #'        The variables in both databases need to match up in content.
-#' @param datahs The data with the information of the independent variables as a disaggregated level.
-#'        Note: The variables and weights used as independent variables must be the same and must have the same name in 'datahp' and in 'datahs'
-#' @param weights A character string specifying the column name to be used as weights in both `datahp` and `datahs` datasets.
-#' If `weights` is provided and exists in both datasets, each dataset's weights will be normalized by the sum of the weights in that dataset.
+#' @param dataB The data which contains information on the independent variables at a disaggregated level.
+#'        Note: The variables and weights used as independent variables must be the same and must have the same name in 'dataA' and in 'dataB'
+#' @param weights A character string specifying the column name to be used as weights in both 'dataA' and 'dataB' datasets.
+#' If the argument `weights` is provided and present in both datasets,  the weights in each dataset will be normalized by the sum of the weights within that dataset.
 #' If `weights` is NULL or the specified column does not exist in both datasets, equal weights are applied across all observations.
 #' @param tol The tolerance to be applied in the optimization function. If the tolerance is not specified, the default tolerance has been set in 1e-10
 #' @param v The support vector
@@ -40,9 +40,10 @@
 #' but can also increases computation time.If the maximum number of iterations is not specified, it will default to 1000
 #' @return The function will provide you a dataframe called table with the next information:
 #'    \itemize{
-#'   \item \strong{probabilities}  Probabilities for each individual to each possibility j of the variable of interest y.
-#'   \item \strong{error primal} Errors calculated to the j possibilities of y.
-#'   \item \strong{predictions} The prediction for each individual is calculated as the sum of the probability plus the error primal.
+#'   \item \strong{weights} The weights used in the optimization process. 
+#'   \item \strong{predictions}  The prediction for each individual is calculated as the sum of the probability plus the error.
+#'   \item \strong{probabilities}  Probabilities for each individual to each possibility \code{j} of the variable of interest \code{y}.
+#'   \item \strong{errors}  Errors calculated to the \code{j} possibilities of \code{y}.
 #'   The function provides information about the optimization process as :
 #'     \item \strong{value_of_entropy} The value of entropy resulting from the optimization.
 #'   \item \strong{iterations} Indicates the times the objective function and the gradient has been evaluated during the optimization process
@@ -56,50 +57,50 @@
 #'   }
 #'   The restriction g3 can be checked thoroughly with the objects by separate.
 #'    \itemize{
-#'   \item  \strong{cross moments hp} Cross moments in \code{datahp}.
-#'   \item  \strong{cross moments hs} Cross moments in \code{datahs}.}
+#'   \item  \strong{cross moments A} Cross moments in \code{dataA}.
+#'   \item  \strong{cross moments B} Cross moments in \code{dataB}.}
 #' @references
 #' Fernandez-Vazquez, E., Díaz-Dapena, A., Rubiera-Morollon, F., Viñuela, A., (2020) Spatial Disaggregation of Social Indicators: An Info-Metrics Approach. Social Indicators Research, 152(2), 809–821. https://doi.org/10.1007/s11205-020-02455-z.
 #' @examples
 #' #In this example we use the data of this package
-#' datahp <- financial()
-#' datahs <- social()
+#' dataA <- financial()
+#' dataB <- social()
 #' # Setting up our function for the dependent variable.
-#' fn               <- datahp$poor_liq ~ Dcollege+Totalincome+Dunemp
-#' #Applying the function ei_gme to our databases. In this case datahp
-#' #is the data where we have our variable of interest datahs is the data
+#' fn               <- dataA$poor_liq ~ Dcollege+Totalincome+Dunemp
+#' #Applying the function ei_gme to our databases. In this case dataA
+#' #is the data where we have our variable of interest dataB is the data
 #' # where we have the information for the disaggregation.
 #' #w can be included if we have weights in both surveys
 #' #Tolerance in this example is fixed in 1e-10 and v will be (1,0,-1)
 #' v=matrix(c(1, 0, -1), nrow = 1)
-#' result  <- ei_gme(fn=fn,datahp=datahp,datahs=datahs,weights="w",v=v)
+#' result  <- ei_gme(fn=fn,dataA=dataA,dataB=dataB,weights="w",v=v)
 #' @export
-ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
+ei_gme <- function(fn,dataA,dataB,weights=NULL,tol,v,iter){
 
   #Independent variables
-  x_hp             <- model.matrix(fn,datahp)                         #Matrix of independent variables in hp - dimensions n_hs (observations in hp) and k (parameters)
+  x_A             <- model.matrix(fn,dataA)                         #Matrix of independent variables in A - dimensions n_B (observations in A) and k (parameters)
 
-  fn_right         <- fn[-2]                                          #hs does not have dependent variable. It is needed a formula without the left hand side
-  x_hs             <- model.matrix(fn_right,datahs)                   #Matrix of independent variables in hs - dimensions n_hp (observations in hs) and k (parameters)
+  fn_right         <- fn[-2]                                          #B does not have dependent variable. It is needed a formula without the left hand side
+  x_B             <- model.matrix(fn_right,dataB)                   #Matrix of independent variables in B - dimensions n_A (observations in B) and k (parameters)
 
-  if (!is.null(weights) && weights %in% colnames(datahp) && weights %in% colnames(datahs)) {
+  if (!is.null(weights) && weights %in% colnames(dataA) && weights %in% colnames(dataB)) {
     # Use the specified column for weights, normalizing by the sum of weights
-    w_hp <- datahp[[weights]] / sum(datahp[[weights]], na.rm = TRUE)
-    w_hs <- datahs[[weights]] / sum(datahs[[weights]], na.rm = TRUE)
+    w_A <- dataA[[weights]] / sum(dataA[[weights]], na.rm = TRUE)
+    w_B <- dataB[[weights]] / sum(dataB[[weights]], na.rm = TRUE)
   } else {
     # If no weights or column is missing, apply equal weights
-    w_hp <- rep(1 / nrow(datahp), nrow(datahp))
-    w_hs <- rep(1 / nrow(datahs), nrow(datahs))
+    w_A <- rep(1 / nrow(dataA), nrow(dataA))
+    w_B <- rep(1 / nrow(dataB), nrow(dataB))
   }
-  if (any(is.na(x_hs))) {
+  if (any(is.na(x_B))) {
     stop("NA (missing) values have been found in the dataset")
   }
-  if (any(is.na(x_hp))) {
+  if (any(is.na(x_A))) {
     stop("NA (missing) values have been found in the dataset")
   }
 
-  #Definition of "y" as a matrix with n_hp observations by j categories.
-  form <- model.frame(fn,datahp)
+  #Definition of "y" as a matrix with n_A observations by j categories.
+  form <- model.frame(fn,dataA)
   y          <- model.response(form) 
   if (length(unique(y)) == 2 && all(sort(unique(y)) == c(0, 1))) {
     y_factor <- factor(y, levels = c(1, 0))  
@@ -108,9 +109,9 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   }
   y_prev <- model.matrix(~ y_factor - 1)
 
-  n_hp             <- dim(x_hp)[1]  ; n_hp                            #observations in hp
-  n_hs             <- dim(x_hs)[1]  ; n_hs                            #Observations in hs
-  k                <- dim(x_hs)[2]  ; k                               #columns in the matrix of independent variables (including the intercept)
+  n_A             <- dim(x_A)[1]  ; n_A                            #observations in A
+  n_B             <- dim(x_B)[1]  ; n_B                            #Observations in B
+  k                <- dim(x_B)[2]  ; k                               #columns in the matrix of independent variables (including the intercept)
   J                <- ncol(y_prev)    ; J                               #categories or columns in y
   #Support vector for each J
 
@@ -136,16 +137,16 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   l                <- dim(v)[2]                                       #Parameter L is equal to the number of values in vector v
   #Auxiliary matrixes
   L_ones           <- c(1,1,1)                                        #A matrix with L ones.    It is used in the restrictions for w
-  N_ones_hs        <- matrix(1,n_hs,1)                                #A matrix with n_hs ones. It is used in the restrictions for p and w
+  N_ones_B        <- matrix(1,n_B,1)                                #A matrix with n_B ones. It is used in the restrictions for p and w
   J_ones           <- matrix(1,J,1)                                   #A matrix with J ones.    It is used in the restrictions for p
 
   #Define the Shannon entropy function (I multiply it by -1 to maximize it)
   shannon_entropy_dual <- function(lambda_v) {
     lambda         <- matrix(lambda_v, nrow=J ,ncol=, byrow=F)        #Lambda is the parameter to be obtained. Here we transform them into matrix form.
-    omega          <- rowSums(t(exp(lambda %*% t(x_hs * w_hs))))     #With J_ones they are multiplied to make the sum in J; t(x_hs * w_hs) this part is Xt(k,n_hs) times the weights of the HS
-    psi            <- t(exp(v[1]*lambda %*% t(x_hs * w_hs)) + exp(v[2]*lambda %*% t(x_hs * w_hs)) + exp(v[3]*lambda %*% t(x_hs * w_hs)))#we do the multiplication in three steps because we have to multiply the support vector by the product of lambda and Xt(k,n_hs) by the weights of the HS, CHECK THAT IT IS EQUIVALENT
+    omega          <- rowSums(t(exp(lambda %*% t(x_B * w_B))))     #With J_ones they are multiplied to make the sum in J; t(x_B * w_B) this part is Xt(k,n_B) times the weights of the B
+    psi            <- t(exp(v[1]*lambda %*% t(x_B * w_B)) + exp(v[2]*lambda %*% t(x_B * w_B)) + exp(v[3]*lambda %*% t(x_B * w_B)))#we do the multiplication in three steps because we have to multiply the support vector by the product of lambda and Xt(k,n_B) by the weights of the B, CHECK THAT IT IS EQUIVALENT
     #Objective function
-    sum(log(omega))  + sum(log(psi)) - sum(t(x_hp) %*%  (y_prev  *  w_hp) * t(lambda))   }
+    sum(log(omega))  + sum(log(psi)) - sum(t(x_A) %*%  (y_prev  *  w_A) * t(lambda))   }
 
 
   lambda_ini       <- matrix(0,J,k)
@@ -176,43 +177,43 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   
   lambda          <- matrix(res$par, nrow=J ,ncol=, byrow=F)
   #Final estimation of  and psi
-  omega       <- rowSums(t(exp(lambda %*% t(x_hs * w_hs))))                                                                          #With J_ones they are multiplied to make the sum in J; t(x_hs * w_hs) this part is Xt(k,n_hs) times the weights of the HS
-  psi         <- t(exp(v[1]*lambda %*% t(x_hs * w_hs)) + exp(v[2]*lambda %*% t(x_hs * w_hs)) + exp(v[3]*lambda %*% t(x_hs * w_hs)))  #we do the multiplication in three steps because we have to multiply the support vector by the product of lambda and Xt(k,n_hs) by the weights of the HS, CHECK THAT IT IS EQUIVALENT
+  omega       <- rowSums(t(exp(lambda %*% t(x_B * w_B))))                                                                          #With J_ones they are multiplied to make the sum in J; t(x_B * w_B) this part is Xt(k,n_B) times the weights of the B
+  psi         <- t(exp(v[1]*lambda %*% t(x_B * w_B)) + exp(v[2]*lambda %*% t(x_B * w_B)) + exp(v[3]*lambda %*% t(x_B * w_B)))  #we do the multiplication in three steps because we have to multiply the support vector by the product of lambda and Xt(k,n_B) by the weights of the B, CHECK THAT IT IS EQUIVALENT
 
   #Estimation of p_dual and errors from lamdbas
-  p_dual      <- t(exp(lambda %*% (t(x_hs * w_hs))))/omega
-  u_dual_1    <- t(exp(v[1]* lambda %*% t(x_hs * w_hs)))/psi
-  u_dual_2    <- t(exp(v[2]* lambda %*% t(x_hs * w_hs)))/psi
-  u_dual_3    <- t(exp(v[3]* lambda %*% t(x_hs * w_hs)))/psi
+  p_dual      <- t(exp(lambda %*% (t(x_B * w_B))))/omega
+  u_dual_1    <- t(exp(v[1]* lambda %*% t(x_B * w_B)))/psi
+  u_dual_2    <- t(exp(v[2]* lambda %*% t(x_B * w_B)))/psi
+  u_dual_3    <- t(exp(v[3]* lambda %*% t(x_B * w_B)))/psi
   error_dual      <- u_dual_1 * v[1] + u_dual_2 * v[2] + u_dual_3 * v[3]
   predictions     <- p_dual+ error_dual
 
   w               <- cbind(u_dual_1,u_dual_2,u_dual_3)
 
   #Restrictions
-  g1              <-  p_dual %*% J_ones - N_ones_hs                        #First  set of restrictions defines the sum of probabilities for each household in hs must be one - n_hs restrictions
-  g2              <-  u_dual_1 + u_dual_2 + u_dual_3 - 1                   #Second set of restrictions defines the sum of probabilities for each household in hs must be one in the errors must be one in each j dimension - n_hs * j restricstions
+  g1              <-  p_dual %*% J_ones - N_ones_B                        #First  set of restrictions defines the sum of probabilities for each household in B must be one - n_B restrictions
+  g2              <-  u_dual_1 + u_dual_2 + u_dual_3 - 1                   #Second set of restrictions defines the sum of probabilities for each household in B must be one in the errors must be one in each j dimension - n_B * j restricstions
 
-  #Cross moments in the hp. "y" is weighted with "w_hp" by a multiplication element by element (NOT a matrix multiplication)
-  cross_moments_hp<- (t(x_hp) %*%  (y_prev  *  w_hp))
+  #Cross moments in the A. "y" is weighted with "w_A" by a multiplication element by element (NOT a matrix multiplication)
+  cross_moments_A<- (t(x_A) %*%  (y_prev  *  w_A))
   error_primal <- error_dual
   
-  #Cross moments in the hs. "y" is weighted with "w_hs" by a multiplication element by element (NOT a matrix multiplication)
-  cross_moments_hs <- (t(x_hs) %*% ((p_dual * w_hs) + ( error_primal  * w_hs)))
+  #Cross moments in the B. "y" is weighted with "w_B" by a multiplication element by element (NOT a matrix multiplication)
+  cross_moments_B <- (t(x_B) %*% ((p_dual * w_B) + ( error_primal  * w_B)))
   
   
-  g3 <- cross_moments_hp - cross_moments_hs  #Last set of restriction defines the cross moments in hp must be equal to hs
-  cross_moments_hs <- noquote(apply(cross_moments_hs, c(1, 2), function(x) sprintf("%10.2f", x)))
-  cross_moments_hp <- noquote(apply(cross_moments_hp, c(1, 2), function(x) sprintf("%10.2f", x)))
+  g3 <- cross_moments_A - cross_moments_B  #Last set of restriction defines the cross moments in A must be equal to B
+  cross_moments_B <- noquote(apply(cross_moments_B, c(1, 2), function(x) sprintf("%10.2f", x)))
+  cross_moments_A <- noquote(apply(cross_moments_A, c(1, 2), function(x) sprintf("%10.2f", x)))
   
   #Set the colnames for the table
-  table <- data.frame(w_hs,predictions, p_dual, error_dual)                                                                        #Joining all the relevant information in one table
+  table <- data.frame(w_B,predictions, p_dual, error_dual)                                                                        #Joining all the relevant information in one table
   category_names <- levels(y_factor)
   assign_col_names <- function(df, y_factor) {
-    w_hs_names <-("weights")
+    w_B_names <-("weights")
     prediction_names <- paste0("predictions_", category_names)  # Usamos los nombres de las categorías
-    p_names <- paste0("p_dual_", category_names)
-    e_names <- paste0("e_dual_", category_names)
+    p_names <- paste0("probabilities_", category_names)
+    e_names <- paste0("errors_", category_names)
     all<- c("weights", prediction_names, p_names,e_names)
     colnames(df) <- all
     return(df)}
@@ -220,8 +221,8 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   
   
   
-  colnames(cross_moments_hs) <- category_names
-  colnames(cross_moments_hp) <- category_names
+  colnames(cross_moments_B) <- category_names
+  colnames(cross_moments_A) <- category_names
   colnames(g3)<-category_names
   table <- assign_col_names(table, y_factor)
 
@@ -255,10 +256,10 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   rownames(table2)<-category_names
   
   
-  q <- matrix(1/J, n_hs, J)
+  q <- matrix(1/J, n_B, J)
   divergencekl <- sum(p_dual * log(p_dual / q))
                  
-  generate_output <- function(estimations, values, tol, v, lambda, checkrestrictions, cross_moments_hp, cross_moments_hs, J, fn,divergencekl) {
+  generate_output <- function(estimations, values, tol, v, lambda, checkrestrictions, cross_moments_A, cross_moments_B, J, fn,divergencekl) {
     output <- structure(list(
       estimations = estimations,
       values = values,
@@ -266,8 +267,8 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
       v = v,
       lambda = lambda,
       checkrestrictions = checkrestrictions,
-      cross_moments_hp = cross_moments_hp,
-      cross_moments_hs = cross_moments_hs,
+      cross_moments_A = cross_moments_A,
+      cross_moments_B = cross_moments_B,
       J = J,
       fn = fn,
       divergencekl=divergencekl
@@ -277,7 +278,7 @@ ei_gme <- function(fn,datahp,datahs,weights=NULL,tol,v,iter){
   row.names(checkrestrictions$g3)<-k_names
   
 
-  output <- generate_output(estimations = table,values =values,tol=tol,v=v, lambda=table2,checkrestrictions=checkrestrictions,cross_moments_hp=cross_moments_hp,cross_moments_hs,J=J,fn=fn,divergencekl=divergencekl)
+  output <- generate_output(estimations = table,values =values,tol=tol,v=v, lambda=table2,checkrestrictions=checkrestrictions,cross_moments_A=cross_moments_A,cross_moments_B,J=J,fn=fn,divergencekl=divergencekl)
 
   class(output) <- "shannon"
   return(output)
